@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-sms.c,v 1.172 2007/11/21 20:32:01 pkot Exp $
+  $Id: gsm-sms.c,v 1.178 2008/02/22 13:26:16 dforsi Exp $
 
   G N O K I I
 
@@ -32,6 +32,8 @@
 
 */
 
+#include "config.h"
+
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -41,6 +43,10 @@
 
 #include "sms-nokia.h"
 
+#ifdef ENABLE_NLS
+#  include <locale.h>
+#endif
+
 #undef ERROR
 #define ERROR() do { if (error != GN_ERR_NONE) return error; } while (0)
 
@@ -337,7 +343,7 @@ static gn_error sms_status(unsigned char status, gn_sms *sms)
 				dprintf("Service rejected");
 				break;
 			case 0x64:
-				dprintf("Quality of service not aviable");
+				dprintf("Quality of service not available");
 				break;
 			case 0x65:
 				dprintf("Error in SME");
@@ -363,7 +369,7 @@ static gn_error sms_status(unsigned char status, gn_sms *sms)
 				dprintf("Not obtainable");
 				break;
 			case 0x44:
-				dprintf("Quality of service not aviable");
+				dprintf("Quality of service not available");
 				break;
 			case 0x45:
 				dprintf("No internetworking available");
@@ -404,7 +410,7 @@ static gn_error sms_status(unsigned char status, gn_sms *sms)
 			dprintf("Service rejected");
 			break;
 		case 0x24:
-			dprintf("Quality of service not aviable");
+			dprintf("Quality of service not available");
 			break;
 		case 0x25:
 			dprintf("Error in SME");
@@ -688,8 +694,9 @@ static gn_error sms_pdu_decode(gn_sms_raw *rawsms, gn_sms *sms)
 			sms->user_data[0].type = GN_SMS_DATA_Bitmap;
 			gn_bmp_sms_read(GN_BMP_PictureMessage, rawsms->user_data,
 					NULL, &sms->user_data[0].u.bitmap);
+#ifdef DEBUG
 			gn_bmp_print(&sms->user_data[0].u.bitmap, stderr);
-
+#endif
 			size = rawsms->user_data_length - 4 - sms->user_data[0].u.bitmap.size;
 			/* Second part is a text */
 			sms->user_data[1].type = GN_SMS_DATA_NokiaText;
@@ -712,7 +719,9 @@ static gn_error sms_pdu_decode(gn_sms_raw *rawsms, gn_sms *sms)
 			gn_bmp_sms_read(GN_BMP_PictureMessage,
 					rawsms->user_data + rawsms->user_data[0] + 7,
 					NULL, &sms->user_data[0].u.bitmap);
+#ifdef DEBUG
 			gn_bmp_print(&sms->user_data[0].u.bitmap, stderr);
+#endif
 		}
 		break;
 	/* Plain text message */
@@ -744,6 +753,163 @@ gn_error gn_sms_parse(gn_data *data)
 	return sms_pdu_decode(data->raw_sms, data->sms);
 }
 
+/**
+ * gn_sms_pdu2raw - Copy PDU data into gn_sms_raw structure
+ * @rawsms: gn_sms_raw structure to be filled
+ * @pdu: buffer containing PDU data
+ * @pdu_len: length of buffer containing PDU data
+ * @flags: GN_SMS_PDU_* flags to control copying
+ *
+ * This function fills a gn_sms_raw structure copying raw values from
+ * a buffer containing PDU data.
+ */
+gn_error gn_sms_pdu2raw(gn_sms_raw *rawsms, unsigned char *pdu, int pdu_len, int flags)
+{
+	gn_error ret = GN_ERR_NONE;
+	unsigned int l, extraoffset, offset = 0;
+
+	if (!flags & GN_SMS_PDU_NOSMSC) {
+		l = pdu[offset] + 1;
+		if (l > pdu_len || l > GN_SMS_SMSC_NUMBER_MAX_LENGTH) {
+			dprintf("Invalid message center length (%d)\n", l);
+			ret = GN_ERR_INTERNALERROR;
+			goto out;
+		}
+		memcpy(rawsms->message_center, pdu, l);
+		offset += l;
+	}
+
+	rawsms->reject_duplicates   = 0;
+	rawsms->report_status       = 0;
+	rawsms->reference           = 0;
+	rawsms->reply_via_same_smsc = 0;
+	rawsms->report              = 0;
+
+	dprintf("rawsms->type: %d\n", rawsms->type);
+
+	rawsms->type                = (pdu[offset] & 0x03) << 1;
+	switch (rawsms->type) {
+	case GN_SMS_MT_Deliver:
+		dprintf("SMS-DELIVER found\n");
+		rawsms->more_messages       = pdu[offset] & 0x04;
+		// bits 3 and 4 of the first octet unused;
+		rawsms->report_status       = pdu[offset] & 0x20;
+		extraoffset = 10;
+		break;
+	case GN_SMS_MT_Submit:
+		dprintf("SMS-SUBMIT found\n");
+		rawsms->reject_duplicates   = pdu[offset] & 0x04;
+		rawsms->validity_indicator  = (pdu[offset] & 0x18) >> 3;
+		extraoffset = 3;
+		break;
+	case GN_SMS_MT_Command:
+		/* TP-Message-Type-Indicator */
+		rawsms->type                = GN_SMS_MT_StatusReport;
+		/* TP-User-Data-Header-Indication */
+		rawsms->udh_indicator       = pdu[offset] & 0x40;
+		/* TP-More-Messages-to-Send */
+		rawsms->more_messages       = pdu[offset] & 0x04;
+		/* TP-Status-Report-Qualifier */
+		rawsms->report              = pdu[offset] & 0x10;
+		offset++;
+		/* TP-Message-Reference */
+		rawsms->reference           = pdu[offset++];
+		/* TP-Recipient-Address */
+		/* TP-Service-Centre-Time-Stamp */
+		/* TP-Discharge-Time */
+		/* TP-Status */
+		/* TP-Parameter-Indicator */
+		/* TP-Protocol-Identifier */
+		/* TP-Data-Coding-Scheme */
+		/* TP-User-Data-Length */
+		/* TP-User-Data */
+		extraoffset = 0;
+#if 0
+		dprintf("SMS-COMMAND found\n");
+		/* TP-Message-Type-Indicator */
+		/** decoded above **/
+		offset = 5;
+		/* TP-User-Data-Header-Indication */
+		rawsms->udh_indicator       = pdu[offset] & 0x40;
+		/* TP-Status-Report-Request */
+		rawsms->report_status       = pdu[offset] & 0x08;
+		offset++;
+		/* TP-Message Reference */
+		rawsms->reference           = pdu[offset++];
+		/* TP-Protocol-Identifier */
+		rawsms->pid                 = pdu[offset++];
+		/* TP-Command-Type */
+		dprintf("TP-Command-Type 0x%02x\n", pdu[offset++]);
+		/* TP-Message-Number */
+		dprintf("TP-Message-Number 0x%02x\n", pdu[offset++]);
+		/* TP-Destination-Address */
+	l = (pdu[offset] % 2) ? pdu[offset] + 1 : pdu[offset];
+	l = l / 2 + 2;
+	memcpy(rawsms->remote_number, pdu + offset, l);
+	offset += l;
+		/* TP-Command-Data-Length */
+		dprintf("TP-Command-Data-Length 0x%02x\n", pdu[offset++]);
+		/* TP-Command-Data */
+		dprintf("TP-Command-Data 0x%02x ...\n", pdu[offset++]);
+		goto out;
+		break;
+#endif
+	default:
+		dprintf("Unknown PDU type %d\n", rawsms->type);
+		return GN_ERR_INTERNALERROR;
+ 	}
+ 	rawsms->more_messages       = pdu[offset];
+	rawsms->udh_indicator       = pdu[offset] & 0x40;
+	offset++;
+
+	if (rawsms->type == GN_SMS_MT_Submit) {
+		rawsms->reference = pdu[offset++];
+	}
+
+	l = (pdu[offset] % 2) ? pdu[offset] + 1 : pdu[offset];
+	l = l / 2 + 2;
+	if (l + offset + extraoffset > pdu_len || l > GN_SMS_NUMBER_MAX_LENGTH) {
+		dprintf("Invalid remote number length (%d)\n", l);
+		ret = GN_ERR_INTERNALERROR;
+		goto out;
+	}
+	memcpy(rawsms->remote_number, pdu + offset, l);
+	offset += l;
+	rawsms->pid                 = pdu[offset++];
+	rawsms->dcs                 = pdu[offset++];
+	if (rawsms->type == GN_SMS_MT_Deliver) {
+		memcpy(rawsms->smsc_time, pdu + offset, 7);
+		offset += 7;
+	}
+	if (rawsms->type == GN_SMS_MT_Submit) {
+		if (rawsms->validity_indicator == GN_SMS_VP_None) {
+			offset += 0;
+		} else if (rawsms->validity_indicator == GN_SMS_VP_RelativeFormat) {
+			// FIXME save this info
+			offset += 1;
+		} else if (rawsms->validity_indicator == GN_SMS_VP_EnhancedFormat ||
+			   rawsms->validity_indicator == GN_SMS_VP_AbsoluteFormat) {
+			// FIXME save this info
+			offset += 7;
+		} else { 
+			dprintf("Internal Error on validity_indicator");
+			return GN_ERR_INTERNALERROR;
+		}
+	}
+	rawsms->length              = pdu[offset++];
+	rawsms->user_data_length = rawsms->length;
+	if (rawsms->udh_indicator & 0x40)
+		rawsms->user_data_length -= pdu[offset] + 1;
+	if (pdu_len - offset > 1000) {
+		dprintf("Phone gave as poisonous (too short?) reply, either phone went crazy or communication went out of sync\n");
+		ret = GN_ERR_INTERNALERROR;
+		goto out;
+	}
+	memcpy(rawsms->user_data, pdu + offset, pdu_len - offset);
+out:
+	return ret;
+}
+
 /**
  * gn_sms_request - High-level function for the explicit SMS reading from the phone
  * @data: GSM data for the phone driver
