@@ -1,6 +1,6 @@
 /*
 
-  $Id: atbus.c,v 1.55 2007/11/22 16:44:17 pkot Exp $
+  $Id: atbus.c,v 1.57 2008/01/09 13:09:45 ladis Exp $
 
   G N O K I I
 
@@ -30,18 +30,15 @@
 
 */
 
+#include "config.h"
+
 /* System header files */
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
 
-#ifdef HAVE_BLUETOOTH
-#  include <bluetooth/bluetooth.h>
-#endif
-
 /* Various header file */
-#include "config.h"
 #include "compat.h"
 #include "misc.h"
 #include "links/atbus.h"
@@ -61,9 +58,9 @@
  * message from the phone.
  */
 
-static int xwrite(unsigned char *d, int len, struct gn_statemachine *sm)
+static int xwrite(unsigned char *d, size_t len, struct gn_statemachine *sm)
 {
-	int res;
+	size_t res;
 
 	at_dprintf("write: ", d, len);
 
@@ -320,14 +317,12 @@ gn_error atbus_initialise(int mode, struct gn_statemachine *state)
 			goto err;
 		}
 		break;
-#ifdef HAVE_BLUETOOTH
 	case GN_CT_Bluetooth:
 		if (!device_open(state->config.port_device, false, false, false, state->config.connection_type, state)) {
 			error = GN_ERR_FAILED;
 			goto err;
 		}
 		break;
-#endif
 	default:
 		dprintf("Device not supported by AT bus\n");
 		error = GN_ERR_FAILED;
