@@ -1,6 +1,6 @@
 /*
 
-  $Id: atgen.c,v 1.166 2007/11/19 14:51:50 pkot Exp $
+  $Id: atgen.c,v 1.179 2008/03/30 13:45:25 dforsi Exp $
 
   G N O K I I
 
@@ -36,6 +36,8 @@
 
 */
 
+#include "config.h"
+
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -65,6 +67,7 @@ static gn_error ReplyMemoryStatus(int messagetype, unsigned char *buffer, int le
 static gn_error ReplyMemoryRange(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
 static gn_error ReplyCallDivert(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
 static gn_error ReplyGetPrompt(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
+static gn_error ReplyGetSMSFolders(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
 static gn_error ReplyGetSMSStatus(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
 static gn_error ReplySendSMS(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
 static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
@@ -95,6 +98,8 @@ static gn_error AT_WritePhonebook(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_DeletePhonebook(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_CallDivert(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_SetPDUMode(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_GetSMSFolders(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_GetSMSFolderStatus(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetSMSStatus(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_SendSMS(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_SaveSMS(gn_data *data, struct gn_statemachine *state);
@@ -147,6 +152,8 @@ static at_function_init_type at_function_init[] = {
 	{ GN_OP_CallDivert,            AT_CallDivert,            ReplyCallDivert },
 	{ GN_OP_AT_SetPDUMode,         AT_SetPDUMode,            Reply },
 	{ GN_OP_AT_Prompt,             NULL,                     ReplyGetPrompt },
+	{ GN_OP_GetSMSFolders,         AT_GetSMSFolders,         ReplyGetSMSFolders },
+	{ GN_OP_GetSMSFolderStatus,    AT_GetSMSFolderStatus,    Reply },
 	{ GN_OP_GetSMSStatus,          AT_GetSMSStatus,          ReplyGetSMSStatus },
 	{ GN_OP_SendSMS,               AT_SendSMS,               ReplySendSMS },
 	{ GN_OP_SaveSMS,               AT_SaveSMS,               ReplySendSMS },
@@ -239,25 +246,8 @@ gn_driver driver_at = {
 	NULL
 };
 
-char *memorynames[] = {
-	"ME", /* Internal memory of the mobile equipment */
-	"SM", /* SIM card memory */
-	"FD", /* Fixed dial numbers */
-	"ON", /* Own numbers */
-	"EN", /* Emergency numbers */
-	"DC", /* Dialled numbers */
-	"RC", /* Received numbers */
-	"MC", /* Missed numbers */
-	"LD", /* Last dialed */
-	"MT", /* combined ME and SIM phonebook */
-	"TA", /* for compatibility only: TA=computer memory */
-	"CB", /* Currently selected memory */
-};
-
-#define NR_MEMORIES (sizeof(memorynames) / sizeof((memorynames)[0]))
-
 typedef struct {
-	char *str;
+	const char *str;
 	at_charset charset;
 } at_charset_map_t;
 
@@ -328,6 +318,58 @@ void at_decode(int charset, char *dst, char *src, int len)
 	}
 }
 
+/* Return the string representing the charset */
+static const char *at_charset2str(at_charset charset)
+{
+	int i;
+
+	for (i = 0; atcharsets[i].str != NULL; i++) {
+		if (atcharsets[i].charset == charset)
+			return atcharsets[i].str;
+	}
+	return NULL;
+}
+
+/* Set the requested charset as the current charset */
+static gn_error at_set_charset(gn_data *data, struct gn_statemachine *state, at_charset charset)
+{
+	at_driver_instance *drvinst = AT_DRVINST(state);
+	gn_data tmpdata;
+	gn_error error;
+
+	if (drvinst->charset == charset)
+		return GN_ERR_NONE;
+
+	/* load the available charsets if they're not already set */
+	if (drvinst->availcharsets == 0) {
+		error = sm_message_send(10, GN_OP_AT_GetCharset, "AT+CSCS=?\r", state);
+		if (error)
+			return error;
+		gn_data_clear(&tmpdata);
+		error = sm_block_no_retry(GN_OP_AT_GetCharset, &tmpdata, state);
+	}
+
+	/* Try to set the requested charset if it's available */
+	if (drvinst->availcharsets & charset) {
+		const char *charset_s;
+		char req[32];
+
+		charset_s = at_charset2str(charset);
+		snprintf(req, sizeof(req), "AT+CSCS=\"%s\"\r", charset_s);
+		error = sm_message_send(strlen(req), GN_OP_Init, req, state);
+		if (error)
+			return error;
+		error = sm_block_no_retry(GN_OP_Init, &tmpdata, state);
+		if (error)
+			return error;
+		drvinst->charset = charset;
+	} else {
+		error = GN_ERR_NOTSUPPORTED;
+	}
+
+	return error;
+}
+
 at_recv_function_type at_insert_recv_function(int type, at_recv_function_type func, struct gn_statemachine *state)
 {
 	at_driver_instance *drvinst = AT_DRVINST(state);
@@ -542,18 +584,21 @@ gn_error at_memory_type_set(gn_memory_type mt, struct gn_statemachine *state)
 	at_driver_instance *drvinst = AT_DRVINST(state);
 	gn_data data;
 	char req[32];
+	const char *memory_name;
+	int len;
 	gn_error ret = GN_ERR_NONE;
 
 	if (mt != drvinst->memorytype) {
-		if (mt >= NR_MEMORIES)
+		memory_name = gn_memory_type2str(mt);
+		if (!memory_name)
 			return GN_ERR_INVALIDMEMORYTYPE;
-		snprintf(req, sizeof(req), "AT+CPBS=\"%s\"\r", memorynames[mt]);
-		ret = sm_message_send(13, GN_OP_Init, req, state);
-		if (ret)
-			return GN_ERR_NOTREADY;
+		len = snprintf(req, sizeof(req), "AT+CPBS=\"%s\"\r", memory_name);
+		ret = sm_message_send(len, GN_OP_Init, req, state);
+		if (ret != GN_ERR_NONE)
+			return ret;
 		gn_data_clear(&data);
 		ret = sm_block_no_retry(GN_OP_Init, &data, state);
-		if (ret)
+		if (ret != GN_ERR_NONE)
 			return ret;
 		drvinst->memorytype = mt;
 
@@ -578,16 +623,19 @@ gn_error AT_SetSMSMemoryType(gn_memory_type mt, struct gn_statemachine *state)
 	at_driver_instance *drvinst = AT_DRVINST(state);
 	gn_data data;
 	char req[32];
+	const char *memory_name;
 	gn_error ret = GN_ERR_NONE;
 
 	if (mt != drvinst->smsmemorytype) {
-		if (mt >= NR_MEMORIES)
+		memory_name = gn_memory_type2str(mt);
+		if (!memory_name)
 			return GN_ERR_INVALIDMEMORYTYPE;
-		gn_data_clear(&data);
-		snprintf(req, sizeof(req), "AT+CPMS=\"%s\"\r", memorynames[mt]);
+		snprintf(req, sizeof(req), "AT+CPMS=\"%s\"\r", memory_name);
 		ret = sm_message_send(13, GN_OP_Init, req, state);
-		if (ret == GN_ERR_NONE)
-			ret = sm_block_no_retry(GN_OP_Init, &data, state);
+		if (ret != GN_ERR_NONE)
+			return ret;
+		gn_data_clear(&data);
+		ret = sm_block_no_retry(GN_OP_Init, &data, state);
 		if (ret == GN_ERR_NONE)
 			drvinst->smsmemorytype = mt;
 	}
@@ -782,6 +830,7 @@ static gn_error AT_GetMemoryStatus(gn_data *data, struct gn_statemachine *state)
 {
 	gn_error ret;
 
+	at_set_charset(data, state, AT_CHAR_GSM);
 	ret = at_memory_type_set(data->memory_status->memory_type,  state);
 	if (ret)
 		return ret;
@@ -806,10 +855,11 @@ static gn_error AT_ReadPhonebook(gn_data *data, struct gn_statemachine *state)
 	char req[32];
 	gn_error ret;
 
-	ret = state->driver.functions(GN_OP_AT_SetCharset, data, state);
+	at_set_charset(data, state, AT_CHAR_GSM);
+	ret = at_memory_type_set(data->phonebook_entry->memory_type, state);
 	if (ret)
 		return ret;
-	ret = at_memory_type_set(data->phonebook_entry->memory_type, state);
+	ret = state->driver.functions(GN_OP_AT_SetCharset, data, state);
 	if (ret)
 		return ret;
 	snprintf(req, sizeof(req), "AT+CPBR=%d\r", data->phonebook_entry->location + drvinst->memoryoffset);
@@ -824,7 +874,8 @@ static gn_error AT_WritePhonebook(gn_data *data, struct gn_statemachine *state)
 	int len, ofs;
 	char req[256], *tmp;
 	gn_error ret;
-	
+
+	at_set_charset(data, state, AT_CHAR_GSM);
 	ret = at_memory_type_set(data->phonebook_entry->memory_type, state);
 	if (ret)
 		return ret;
@@ -932,12 +983,54 @@ static gn_error AT_SetPDUMode(gn_data *data, struct gn_statemachine *state)
 	return sm_block_no_retry(GN_OP_AT_SetPDUMode, data, state);
 }
 
+static gn_error AT_GetSMSFolders(gn_data *data, struct gn_statemachine *state)
+{
+	gn_error ret;
+
+	if (!data || !data->sms_folder_list)
+		return GN_ERR_INTERNALERROR;
+
+	ret = sm_message_send(10, GN_OP_GetSMSFolders, "AT+CPMS=?\r", state);
+	if (ret != GN_ERR_NONE)
+		return ret;
+	return sm_block_no_retry(GN_OP_GetSMSFolders, data, state);
+}
+
+static gn_error AT_GetSMSFolderStatus(gn_data *data, struct gn_statemachine *state)
+{
+	gn_sms_status smsstatus = {0, 0, 0, 0}, *save_smsstatus;
+	gn_memory_status memory_status = {0, 0, 0}, *save_memory_status;
+	gn_error ret;
+
+	memory_status.memory_type = data->sms_folder->folder_id;
+
+	/* this driver needs some structures that other drivers don't need
+	   and the callers (eg. gnokii) may not be aware of that
+	   so always use a local copy
+	 */
+	save_smsstatus = data->sms_status;
+	data->sms_status = &smsstatus;
+	save_memory_status = data->memory_status;
+	data->memory_status = &memory_status;
+	ret = state->driver.functions(GN_OP_GetSMSStatus, data, state);
+	data->memory_status = save_memory_status;
+	data->sms_status = save_smsstatus;
+	if (ret != GN_ERR_NONE)
+		return ret;
+
+	data->sms_folder->number = smsstatus.number;
+
+	return GN_ERR_NONE;
+}
+
 static gn_error AT_GetSMSStatus(gn_data *data, struct gn_statemachine *state)
 {
 	gn_error ret;
 
 	if (!data->sms_status) return GN_ERR_INTERNALERROR;
 
+	at_set_charset(data, state, AT_CHAR_GSM);
+
         if (data->memory_status) {
                 ret = AT_SetSMSMemoryType(data->memory_status->memory_type,  state);
                 if (ret != GN_ERR_NONE)
@@ -957,7 +1050,10 @@ static gn_error AT_SendSMS(gn_data *data, struct gn_statemachine *state)
 
 static gn_error AT_SaveSMS(gn_data *data, struct gn_statemachine *state)
 {
-	gn_error ret = AT_SetSMSMemoryType(data->raw_sms->memory_type,  state);
+	gn_error ret;
+	
+	at_set_charset(data, state, AT_CHAR_GSM);
+	ret = AT_SetSMSMemoryType(data->raw_sms->memory_type,  state);
 	if (ret)
 		return ret;
 	return AT_WriteSMS(data, state, "CMGW");
@@ -1044,7 +1140,10 @@ static gn_error AT_GetSMS(gn_data *data, struct gn_statemachine *state)
 {
 	/* Sony Ericsson can return on notification a location that is 9-digits long */
 	unsigned char req[32];
-	gn_error err = AT_SetSMSMemoryType(data->raw_sms->memory_type,  state);
+	gn_error err;
+
+	at_set_charset(data, state, AT_CHAR_GSM);
+	err = AT_SetSMSMemoryType(data->raw_sms->memory_type,  state);
 
 	if (err)
 		return err;
@@ -1065,7 +1164,10 @@ static gn_error AT_GetSMS(gn_data *data, struct gn_statemachine *state)
 static gn_error AT_DeleteSMS(gn_data *data, struct gn_statemachine *state)
 {
 	unsigned char req[32];
-	gn_error err = AT_SetSMSMemoryType(data->raw_sms->memory_type,  state);
+	gn_error err;
+	
+	at_set_charset(data, state, AT_CHAR_GSM);
+	err = AT_SetSMSMemoryType(data->raw_sms->memory_type,  state);
 	if (err)
 		return err;
 	snprintf(req, sizeof(req), "AT+CMGD=%d\r", data->sms->number);
@@ -1084,6 +1186,9 @@ static gn_error AT_DeleteSMS(gn_data *data, struct gn_statemachine *state)
  */
 static gn_error AT_GetSMSCenter(gn_data *data, struct gn_statemachine *state)
 {
+	/* AT protocol supports only one SMS Center */
+	if (data->message_center && data->message_center->id != 1)
+		return GN_ERR_INVALIDLOCATION;
  	if (sm_message_send(9, GN_OP_GetSMSCenter, "AT+CSCA?\r", state))
 		return GN_ERR_NOTREADY;
 	return sm_block_no_retry(GN_OP_GetSMSCenter, data, state);
@@ -1316,7 +1421,7 @@ static gn_error ReplyReadPhonebook(int messagetype, unsigned char *buffer, int l
 {
 	at_driver_instance *drvinst = AT_DRVINST(state);
 	at_line_buffer buf;
-	char *pos, *endpos;
+	char *pos;
 	gn_error error;
 
 	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE)
@@ -1342,31 +1447,75 @@ static gn_error ReplyReadPhonebook(int messagetype, unsigned char *buffer, int l
 		return GN_ERR_NONE;
 	}
 	if (data->phonebook_entry) {
+		char *part[6];
+		int parts, quoted;
+
 		data->phonebook_entry->caller_group = GN_PHONEBOOK_GROUP_None;
 		data->phonebook_entry->subentries_count = 0;
 		data->phonebook_entry->empty = false;
 
-		/* store number */
-		pos = strchr(buf.line2, '\"');
-		dprintf("NUMBER: %s\n", pos);
-		endpos = NULL;
-		if (pos)
-			endpos = strchr(++pos, '\"');
-		if (endpos) {
-			*endpos = '\0';
-			snprintf(data->phonebook_entry->number, sizeof(data->phonebook_entry->number), "%s", pos);
+		/* split in several parts a line like +CPBR: 1,"123",129,"Name","2008/03/19,18:57"
+		   part[] must have one more item than the string, otherwise the last *part will point to string tail
+		   commas can appear unescaped inside quotes
+		   quotes can appear only as delimiters
+		 */
+		part[0] = pos = buf.line2 + 7;
+		for (parts = 1; parts < (sizeof(part) / sizeof(*part)); parts++) part[parts] = NULL;
+		quoted = 0;
+		parts = 1;
+		while (*pos && (parts < (sizeof(part) / sizeof(*part)))) {
+			switch (*pos) {
+			case '"':
+				quoted = !quoted;
+				break;
+			case ',':
+				if (!quoted) {
+					*pos = '\0';
+					part[parts] = pos + 1;
+					parts++;
+				}
+				break;
+			}
+			pos++;
+		}
+		/* DEBUG */
+		for (parts=0; part[parts]; parts++) {
+			dprintf("part[%d] = \"%s\"\n", parts, part[parts]);
 		}
 
-		/* store name */
-		pos = NULL;
-		if (endpos)
-			pos = strchr(endpos+2, ',');
-		dprintf("NAME: %s\n", pos);
-		endpos = NULL;
+		/* store number */
+		pos = part[1];
 		if (pos) {
-			pos = strip_quotes(pos+1);
+			dprintf("NUMBER: %s\n", pos);
+			pos = strip_quotes(pos);
+			snprintf(data->phonebook_entry->number, sizeof(data->phonebook_entry->number), "%s", pos);
+		}
+		/* store name */
+		pos = part[3];
+		if (pos) {
+			dprintf("NAME: %s\n", pos);
+			pos = strip_quotes(pos);
 			at_decode(drvinst->charset, data->phonebook_entry->name, pos, strlen(pos));
 		}
+		/* store date/time (usually sent with DC, MC, RC entries) */
+		pos = part[4];
+		if (pos) {
+			dprintf("DATE: %s\n", pos);
+			data->phonebook_entry->date.year = 0;
+			data->phonebook_entry->date.month = 0;
+			data->phonebook_entry->date.day = 0;
+			data->phonebook_entry->date.hour = 0;
+			data->phonebook_entry->date.minute = 0;
+			data->phonebook_entry->date.second = 0;
+			/* seconds may not be present */
+			sscanf(pos, "\"%d/%d/%d,%d:%d:%d\"",
+				&data->phonebook_entry->date.year,
+				&data->phonebook_entry->date.month,
+				&data->phonebook_entry->date.day,
+				&data->phonebook_entry->date.hour,
+				&data->phonebook_entry->date.minute,
+				&data->phonebook_entry->date.second);
+		}
 	}
 	return GN_ERR_NONE;
 }
@@ -1602,6 +1751,46 @@ static gn_error ReplyGetPrompt(int messagetype, unsigned char *buffer, int lengt
 	}
 }
 
+static gn_error ReplyGetSMSFolders(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
+{
+	at_line_buffer buf;
+	gn_error error;
+	char *pos, *memory_name;
+	char **items;
+	int i;
+	gn_memory_type memory_type;
+
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
+
+	buf.line1 = buffer + 1;
+	buf.length = length;
+	splitlines(&buf);
+
+	if (strncmp("+CPMS:", buf.line2, 6))
+		return GN_ERR_INTERNALERROR;
+
+	/* split a string like +CPMS: ("ME","SM"),("ME","SM"),("ME","SM")
+	   can't use strip_brackets() because it will strip the last ')', not the first
+	 */
+	pos = buf.line2 + 6;
+	while (*pos && *pos != ')')
+		pos++;
+	*pos = '\0';
+	items = gnokii_strsplit(buf.line2 + 8, ",", 4);
+	for (i = 0; items[i]; i++) {
+		memory_name = strip_quotes(items[i]);
+		memory_type = gn_str2memory_type(memory_name);
+
+		data->sms_folder_list->folder_id[i] = memory_type;
+		data->sms_folder_list->folder[i].folder_id = memory_type;
+		snprintf(data->sms_folder_list->folder[i].name, sizeof(data->sms_folder_list->folder[0].name), "%s", gn_memory_type_print(memory_type));
+	}
+	data->sms_folder_list->number = i;
+	gnokii_strfreev(items);
+
+	return GN_ERR_NONE;
+}
+
 static gn_error ReplyGetSMSStatus(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
 	at_line_buffer buf;
@@ -1634,13 +1823,17 @@ static gn_error ReplySendSMS(int messagetype, unsigned char *buffer, int length,
 	buf.length = length;
 	splitlines(&buf);
 
-	/* SendSMS or SaveSMS */
-	if (!strncmp("+CMGW:", buf.line2, 6) ||
-	    !strncmp("+CMGS:", buf.line2, 6))
+	if (!strncmp("+CMGW:", buf.line2, 6)) {
+		/* SaveSMS */
+		data->raw_sms->number = atoi(buf.line2 + 6);
+		dprintf("Message saved (location: %d)\n", data->raw_sms->number);
+	} else if (!strncmp("+CMGS:", buf.line2, 6)) {
+		/* SendSMS */
 		data->raw_sms->reference = atoi(buf.line2 + 6);
-	else
+		dprintf("Message sent (reference: %d)\n", data->raw_sms->reference);
+	} else {
 		data->raw_sms->reference = -1;
-	dprintf("Message sent (reference: %d)\n", data->raw_sms->reference);
+	}
 	return GN_ERR_NONE;
 }
 
@@ -1648,7 +1841,7 @@ static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length,
 {
 	at_line_buffer buf;
 	gn_error ret = GN_ERR_NONE;
-	unsigned int sms_len, l, extraoffset, offset = 0;
+	unsigned int sms_len, pdu_flags;
 	unsigned char *tmp;
 	gn_error error;
  	at_driver_instance *drvinst = AT_DRVINST(state);
@@ -1732,90 +1925,13 @@ static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length,
 	dprintf("%s\n", buf.line3);
 	hex2bin(tmp, buf.line3, sms_len);
 
-	if(!drvinst->no_smsc) {
-		l = tmp[offset] + 1;
-		if (l > sms_len || l > GN_SMS_SMSC_NUMBER_MAX_LENGTH) {
-			dprintf("Invalid message center length (%d)\n", l);
-			ret = GN_ERR_INTERNALERROR;
-			goto out;
-		}
-		memcpy(data->raw_sms->message_center, tmp, l);
-		offset += l;
-	}
-
-	data->raw_sms->reject_duplicates   = 0;
-	data->raw_sms->report_status       = 0;
-	data->raw_sms->reference           = 0;
-	data->raw_sms->reply_via_same_smsc = 0;
-	data->raw_sms->report              = 0;
-
-	data->raw_sms->type                = (tmp[offset] & 0x03) << 1;
-	if (data->raw_sms->type == GN_SMS_MT_Deliver) { 
-		dprintf("SMS-DELIVER found\n");
-		data->raw_sms->more_messages       = tmp[offset] & 0x04;
-		// bits 3 and 4 of the first octet unused;
-		data->raw_sms->report_status       = tmp[offset] & 0x20;
-		extraoffset = 10;
-	} else if (data->raw_sms->type == GN_SMS_MT_Submit) {
-		dprintf("SMS-SUBMIT found\n");
-		data->raw_sms->reject_duplicates   = tmp[offset] & 0x04;
-		data->raw_sms->validity_indicator  = (tmp[offset] & 0x18) >> 3;
-		extraoffset = 3;
+	if (drvinst->no_smsc) {
+		pdu_flags = GN_SMS_PDU_NOSMSC;
 	} else {
-		dprintf("Unknown PDU type %d\n", data->raw_sms->type);
-		return GN_ERR_INTERNALERROR;
- 	}
- 	data->raw_sms->more_messages       = tmp[offset];
-	data->raw_sms->udh_indicator       = tmp[offset] & 0x40;
-	offset++;
+		pdu_flags = GN_SMS_PDU_DEFAULT;
+	}
+	ret = gn_sms_pdu2raw(data->raw_sms, tmp, sms_len, pdu_flags);
 
-	if (data->raw_sms->type == GN_SMS_MT_Submit) {
-		data->raw_sms->reference = tmp[offset++];
-	} else {
-		data->raw_sms->reference = 0;
-	}
-
-	l = (tmp[offset] % 2) ? tmp[offset] + 1 : tmp[offset];
-	l = l / 2 + 2;
-	if (l + offset + extraoffset > sms_len || l > GN_SMS_NUMBER_MAX_LENGTH) {
-		dprintf("Invalid remote number length (%d)\n", l);
-		ret = GN_ERR_INTERNALERROR;
-		goto out;
-	}
-	memcpy(data->raw_sms->remote_number, tmp + offset, l);
-	offset += l;
-	data->raw_sms->pid                 = tmp[offset++];
-	data->raw_sms->dcs                 = tmp[offset++];
-	if (data->raw_sms->type == GN_SMS_MT_Deliver) {
-		memcpy(data->raw_sms->smsc_time, tmp + offset, 7);
-		offset += 7;
-	}
-	if (data->raw_sms->type == GN_SMS_MT_Submit) {
-		if (data->raw_sms->validity_indicator == GN_SMS_VP_None) {
-			offset += 0;
-		} else if (data->raw_sms->validity_indicator == GN_SMS_VP_RelativeFormat) {
-			// FIXME save this info
-			offset += 1;
-		} else if (data->raw_sms->validity_indicator == GN_SMS_VP_EnhancedFormat ||
-			   data->raw_sms->validity_indicator == GN_SMS_VP_AbsoluteFormat) {
-			// FIXME save this info
-			offset += 7;
-		} else { 
-			dprintf("Internal Error on validity_indicator");
-			return GN_ERR_INTERNALERROR;
-		}
-	}
-	data->raw_sms->length              = tmp[offset++];
-	data->raw_sms->user_data_length = data->raw_sms->length;
-	if (data->raw_sms->udh_indicator & 0x40)
-		data->raw_sms->user_data_length -= tmp[offset] + 1;
-	if (sms_len - offset > 1000) {
-		dprintf("Phone gave as poisonous (too short?) reply %s, either phone went crazy or communication went out of sync\n", buf.line3);
-		ret = GN_ERR_INTERNALERROR;
-		goto out;
-	}
-	memcpy(data->raw_sms->user_data, tmp + offset, sms_len - offset);
-out:
 	free(tmp);
 	return ret;
 }
@@ -1930,6 +2046,8 @@ static gn_error ReplyRing(int messagetype, unsigned char *buffer, int length, gn
 		pos = buf.line1 + 8;
 		if (!strncmp(pos, "VOICE", 5))
 			cinfo.type = GN_CALL_Voice;
+		else if (*pos < ' ') /* some phones reply with "+CRING: <cr><lf>" only */
+			cinfo.type = GN_CALL_Voice;
 		else
 			return GN_ERR_UNHANDLEDFRAME;
 		status = GN_CALL_Incoming;
@@ -2085,7 +2203,7 @@ static gn_error ReplyIncomingSMS(int messagetype, unsigned char *buffer, int len
 	at_driver_instance *drvinst = AT_DRVINST(state);
 	at_line_buffer buf;
 	char *memory, *pos;
-	int index, i;
+	int index;
 	gn_memory_type mem;
 	int freesms = 0;
 	gn_error error = GN_ERR_NONE;
@@ -2112,17 +2230,11 @@ static gn_error ReplyIncomingSMS(int messagetype, unsigned char *buffer, int len
 	memory = strip_quotes(buf.line1 + 7);
 	if (memory == NULL)
 		return GN_ERR_UNSOLICITED;
-	for (i = 0; i < NR_MEMORIES; i++) {
-		if (!strcmp(memory, memorynames[i])) {
-			mem = i;
-			break;
-		}
-	}
-
+	mem = gn_str2memory_type(memory);
 	if (mem == GN_MT_XX)
 		return GN_ERR_UNSOLICITED;
 
-	dprintf("Received message folder %s index %d\n", memorynames[mem], index);
+	dprintf("Received message folder %s index %d\n", gn_memory_type2str(mem), index);
 
 	if (!data->sms) {
 		freesms = 1;
@@ -2156,7 +2268,7 @@ static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int
 	at_line_buffer buf;
 	char *pos;
 	char **strings;
-	gn_error error;
+	gn_error error = GN_ERR_NONE;
 	int i;
 
 	if (!data->network_info)
@@ -2215,8 +2327,16 @@ static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int
 
 		memset(tmp, 0, sizeof(tmp));
 		strings = gnokii_strsplit(buf.line2, ",", 3);
-		format = atoi(strings[1]);
+		/* phone may respond with only the first value as in "+COPS: 0" */
+		if (strings[1]) {
+			format = atoi(strings[1]);
+		} else {
+			format = -1;
+		}
 		switch (format) {
+		case -1: /* neither operator name nor code given (eg. no SIM or not registered) */
+			error = GN_ERR_NOTAVAILABLE;
+			break;
 		case 0: /* network operator name given */
 			pos = strip_quotes(strings[2]);
 			at_decode(drvinst->charset, tmp, pos, strlen(pos));
@@ -2243,13 +2363,14 @@ static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int
 				snprintf(data->network_info->network_code, sizeof(data->network_info->network_code), strings[2]);
 			}
 			break;
-		default: /* FIXME: add error handling */
+		default: /* defined formats are in range (0-2) */
+			error = GN_ERR_UNHANDLEDFRAME;
 			break;
 		}
 		gnokii_strfreev(strings);
 	}
 
-	return GN_ERR_NONE;
+	return error;
 }
 
 static gn_error ReplyGetDateTime(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
