@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii-file.c,v 1.7 2007/08/05 18:14:59 pkot Exp $
+  $Id: gnokii-file.c,v 1.11 2007/11/07 18:28:20 pkot Exp $
 
   G N O K I I
 
@@ -213,7 +213,7 @@ int deletefile(char *filename, gn_data *data, struct gn_statemachine *state)
 	if ((error = gn_sm_functions(GN_OP_DeleteFile, data, state)) != GN_ERR_NONE)
 		fprintf(stderr, _("Failed to delete %s: %s\n"), filename, gn_error_print(error));
 	else {
-		fprintf(stdout, _("Deleted: %s\n"), filename);
+		fprintf(stderr, _("Deleted: %s\n"), filename);
 	}
 	return error;
 }
@@ -233,7 +233,7 @@ int deletefilebyid(char *id, gn_data *data, struct gn_statemachine *state)
 	if ((error = gn_sm_functions(GN_OP_DeleteFileById, data, state)) != GN_ERR_NONE)
 		fprintf(stderr, _("Failed to delete %s: %s\n"), id, gn_error_print(error));
 	else {
-		fprintf(stdout, _("Deleted: %s\n"), id);
+		fprintf(stderr, _("Deleted: %s\n"), id);
 	}
 	return error;
 }
@@ -255,6 +255,7 @@ int getfile(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 	if (argc < optind)
 		getfile_usage(stderr, -1);
 
+	memset(filename2, 0, 512);
 	memset(&fi, 0, sizeof(fi));
 	snprintf(fi.name, 512, "%s", optarg);
 
@@ -266,30 +267,33 @@ int getfile(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 	else {
 		if (argc == optind) {
 			if (strrchr(optarg, '/'))
-				strncpy(filename2, strrchr(optarg, '/') + 1, 512);
+				snprintf(filename2, sizeof(filename2), "%s", strrchr(optarg, '/') + 1);
 			else if (strrchr(optarg, '\\'))
-				strncpy(filename2, strrchr(optarg, '\\') + 1, 512);
+				snprintf(filename2, sizeof(filename2), "%s", strrchr(optarg, '\\') + 1);
 			else
-				strcpy(filename2, "default.dat");
+				snprintf(filename2, sizeof(filename2), "default.dat");
 			fprintf(stdout, _("Got file %s.  Save to [%s]: "), optarg, filename2);
 			gn_line_get(stdin, filename2, 512);
 			if (filename2[0] == 0) {
 				if (strrchr(optarg, '/'))
-					strncpy(filename2, strrchr(optarg, '/') + 1, 512);
+					snprintf(filename2, sizeof(filename2), "%s", strrchr(optarg, '/') + 1);
 				else if (strrchr(optarg, '\\'))
-					strncpy(filename2, strrchr(optarg, '\\') + 1, 512);
+					snprintf(filename2, sizeof(filename2), "%s", strrchr(optarg, '\\') + 1);
 				else
-					strcpy(filename2, "default.dat");
+					snprintf(filename2, sizeof(filename2), "default.dat");
 			}
-			f = fopen(filename2, "w");
 		} else {
-			f = fopen(argv[optind], "w");
+			snprintf(filename2, sizeof(filename2), "%s", argv[optind]);
 		}
+		f = fopen(filename2, "w");
 		if (!f) {
 			fprintf(stderr, _("Can't open file %s for writing!\n"), filename2);
 			return GN_ERR_FAILED;
 		}
-		fwrite(fi.file, fi.file_length, 1, f);
+		if (fwrite(fi.file, fi.file_length, 1, f) < 0) {
+			fprintf(stderr, _("Failed to write to file %s.\n"), filename2);
+			error = GN_ERR_FAILED;
+		}
 		fclose(f);
 		free(fi.file);
 	}
@@ -314,6 +318,7 @@ int getfilebyid(int argc, char *argv[], gn_data *data, struct gn_statemachine *s
 	if (argc < optind)
 		getfilebyid_usage(stderr, -1);
 
+	memset(filename2, 0, 512);
 	memset(&fi, 0, sizeof(fi));
 	set_fileid(&fi, optarg);
 
@@ -329,16 +334,19 @@ int getfilebyid(int argc, char *argv[], gn_data *data, struct gn_statemachine *s
 		error = gn_sm_functions(GN_OP_GetFileById, data, state);
 		if (error == GN_ERR_NONE) {
 			if (argc == optind) {
-				strncpy(filename2, fi.name, 512);
-				f = fopen(filename2, "w");
+				snprintf(filename2, sizeof(filename2), "%s", fi.name);
 			} else {
-				f = fopen(argv[optind], "w");
+				snprintf(filename2, sizeof(filename2), "%s", argv[optind]);
 			}
+			f = fopen(filename2, "w");
 			if (!f) {
 				fprintf(stderr, _("Can't open file %s for writing!\n"), filename2);
 				return GN_ERR_FAILED;
 			}
-			fwrite(fi.file, fi.file_length, 1, f);
+			if (fwrite(fi.file, fi.file_length, 1, f) < 0) {
+				fprintf(stderr, _("Failed to write to file %s.\n"), filename2);
+				error = GN_ERR_FAILED;
+			}
 			fclose(f);
 			free(fi.file);
 		} else {
@@ -375,18 +383,22 @@ int getallfiles(char *path, gn_data *data, struct gn_statemachine *state)
 
 		for (i = 0; i < fi.file_count; i++) {
 			data->file = fi.files[i];
-			strncpy(filename2, fi.files[i]->name, 512);
-			snprintf(fi.files[i]->name, 512, "%s%s", path, filename2);
+			snprintf(filename2, sizeof(filename2), "%s", fi.files[i]->name);
+			snprintf(fi.files[i]->name, sizeof(fi.files[i]->name), "%s%s", path, filename2);
 			if ((error = gn_sm_functions(GN_OP_GetFile, data, state)) != GN_ERR_NONE)
 				fprintf(stderr, _("Failed to get file %s: %s\n"), data->file->name, gn_error_print(error));
 			else {
-				fprintf(stdout, _("Got file %s.\n"), filename2);
+				fprintf(stderr, _("Got file %s.\n"), filename2);
 				f = fopen(filename2, "w");
 				if (!f) {
 					fprintf(stderr, _("Can't open file %s for writing!\n"), filename2);
 					return GN_ERR_FAILED;
 				}
-				fwrite(data->file->file, data->file->file_length, 1, f);
+				if (fwrite(data->file->file, data->file->file_length, 1, f) < 0) {
+					fprintf(stderr, _("Failed to write to file %s.\n"), filename2);
+					fclose(f);
+					return GN_ERR_FAILED; 
+				}
 				fclose(f);
 				free(data->file->file);
 			}
