@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii-sms.c,v 1.36 2007/11/07 18:28:20 pkot Exp $
+  $Id: gnokii-sms.c,v 1.39 2008/03/14 21:39:42 dforsi Exp $
 
   G N O K I I
 
@@ -1058,7 +1058,7 @@ int getsmsc(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 		start = gnokii_atoi(argv[optind]);
 		if (errno || start < 0)
 			getsmsc_usage(stderr, -1);
-		stop = (argc > optind+1) ? gnokii_atoi(argv[optind+1]) : start;
+		stop = parse_end_value_option(argc, argv, optind + 1, start);
 		if (errno || stop < 0)
 			getsmsc_usage(stderr, -1);
 
@@ -1068,7 +1068,7 @@ int getsmsc(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 		}
 
 		if (start < 1) {
-			fprintf(stderr, _("SMS center number must be greater than 1\n"));
+			fprintf(stderr, _("SMS center number must be greater than 0\n"));
 			return -1;
 		}
 	} else {
@@ -1088,6 +1088,9 @@ int getsmsc(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 		case GN_ERR_NONE:
 			break;
 		default:
+			/* ignore the error when reading until "end" and at least one entry has alreadly been read */
+			if ((error == GN_ERR_INVALIDLOCATION) && (stop == INT_MAX) && (i > start))
+				return GN_ERR_NONE;
 			fprintf(stderr, _("Error: %s\n"), gn_error_print(error));
 			return error;
 		}
@@ -1315,7 +1318,7 @@ int showsmsfolderstatus(gn_data *data, struct gn_statemachine *state)
 			fprintf(stderr, _("Cannot stat folder \"%s\": %s\n"), folders.folder[i].name, gn_error_print(error));
 			return error;
 		}
-		fprintf(stdout, _("%3d %-32s %2u %4u\n"), i, folders.folder[i].name, folders.folder_id[i], folders.folder[i].number);
+		fprintf(stdout, _("%3d %-32s %2s %4u\n"), i, folders.folder[i].name, gn_memory_type2str(folders.folder_id[i]), folders.folder[i].number);
 	}
 
 	return GN_ERR_NONE;
