@@ -1,6 +1,6 @@
 /*
 
-  $Id: compat.h,v 1.58 2007/10/16 11:43:59 dforsi Exp $
+  $Id: compat.h,v 1.63 2008/01/11 18:45:54 pkot Exp $
 
   G N O K I I
 
@@ -34,20 +34,20 @@
 #ifndef	_gnokii_compat_h
 #define	_gnokii_compat_h
 
-#include "config.h"
-
 #include <stdlib.h>
 #include <stdio.h>
 
 #ifdef WIN32
 #  include <windows.h>
 #  include <locale.h>
-#  ifdef HAVE_DIRECT_H
-#    include <direct.h>
-#  endif
-#  ifdef HAVE_SYS_STAT_H
-#    include <sys/stat.h>
-#  endif
+#endif
+
+#ifdef HAVE_DIRECT_H
+#  include <direct.h>
+#endif
+
+#ifdef HAVE_SYS_STAT_H
+#  include <sys/stat.h>
 #endif
 
 #ifdef HAVE_STDARG_H
@@ -229,14 +229,14 @@ time_t timegm(struct tm *tm);
 #endif
 
 #ifdef WIN32
-#  define mkdir(dirname, accessrights)	_mkdir(dirname)
-/*
- * This is inspired by Marcin Wiacek <marcin-wiacek@topnet.pl>, it should help
- * windows compilers (MS VC 6)
- */
-#  define inline /* Not supported */
-#  define strcasecmp stricmp
-#  define strncasecmp strnicmp
+#  ifdef _MSC_VER
+#    define inline __inline
+#    define mkdir(dirname, accessrights) _mkdir(dirname)
+#    define strcasecmp _stricmp
+#    define strncasecmp _strnicmp
+#    define snprintf _snprintf
+#    define __const const
+#  endif /* _MSC_VER */
 #  if !defined(HAVE_UNISTD_H) || defined(__MINGW32__)
 #    define sleep(x) Sleep((x) * 1000)
 #    define usleep(x) Sleep(((x) < 1000) ? 1 : ((x) / 1000))
