@@ -1,6 +1,6 @@
 /*
 
-  $Id: nk6510.h,v 1.30 2006/04/11 21:31:42 pkot Exp $
+  $Id: nk6510.h,v 1.33 2007/07/07 12:21:21 pkot Exp $
 
   G N O K I I
 
@@ -82,11 +82,12 @@ typedef enum {
 #define NK6510_SUBSMS_SMS_SEND_FAIL	0x01	/* SMS send failed */
 #define NK6510_SUBSMS_SMS_RCVD		0x10	/* SMS received */
 #define NK6510_SUBSMS_SMSC_RCV		0x15	/* SMS center info received */
-#define NK6510_SUBSMS_CELLBRD_OK	0x21	/* Set cell broadcast success*/
+#define NK6510_SUBSMS_CELLBRD_OK	0x21	/* Set cell broadcast success */
 #define NK6510_SUBSMS_CELLBRD_FAIL	0x22	/* Set cell broadcast failure */
 #define NK6510_SUBSMS_READ_CELLBRD	0x23	/* Read cell broadcast */
-#define NK6510_SUBSMS_SMSC_OK		0x31	/* Set SMS center success*/
+#define NK6510_SUBSMS_SMSC_OK		0x31	/* Set SMS center success */
 #define NK6510_SUBSMS_SMSC_FAIL		0x32	/* Set SMS center failure */
+#define NK6510_SUBSMS_INCOMING		0x04	/* Incoming SMS notification */
 
 /* Clock handling message subtypes (send) */
 #define NK6510_SUBCLO_GET_DATE		0x0a	/* Get date & time */
@@ -110,17 +111,17 @@ typedef enum {
 #define NK6510_SUBCAL_DEL_NOTE		0x0b	/* Delete note */
 #define NK6510_SUBCAL_GET_NOTE		0x19	/* Get note */
 #define NK6510_SUBCAL_GET_FREEPOS	0x31	/* Get first free position */
-#define NK6510_SUBCAL_GET_INFO		0x3a	/* Calendar sumary */
+#define NK6510_SUBCAL_GET_INFO		0x3a	/* Calendar summary */
 /* Calendar handling message subtypes (recv) */
 #define NK6510_SUBCAL_ADD_MEETING_RESP	0x02	/* Add meeting note response */
 #define NK6510_SUBCAL_ADD_CALL_RESP	0x04	/* Add call note response */
 #define NK6510_SUBCAL_ADD_BIRTHDAY_RESP	0x06	/* Add birthday note response */
 #define NK6510_SUBCAL_ADD_REMINDER_RESP	0x08	/* Add reminder note response */
-#define NK6510_SUBCAL_DEL_NOTE_RESP	0x0c	/* Dletete note response */
+#define NK6510_SUBCAL_DEL_NOTE_RESP	0x0c	/* Delete note response */
 #define NK6510_SUBCAL_NOTE_RCVD		0x1a	/* Received note */
 #define NK6510_SUBCAL_FREEPOS_RCVD	0x32	/* Received first free position */
 #define NK6510_SUBCAL_INFO_RCVD		0x3b	/* Received calendar summary */
-#define NK6510_SUBCAL_NOTE2_RCVD	0x7e	/* Received note (with more detailes) */
+#define NK6510_SUBCAL_NOTE2_RCVD	0x7e	/* Received note (with more details) */
 #define NK6510_SUBCAL_ADD_NOTE_RESP	0x66	/* Add calendar note response */
 
 /* Calendar note types */
@@ -198,4 +199,16 @@ typedef enum {
 
 #define NK6510_FILE_ID_LENGTH 6
 
+typedef struct {
+	/* callbacks */
+	void (*on_cell_broadcast)(gn_cb_message *msg, struct gn_statemachine *state, void *callback_data);
+	void (*call_notification)(gn_call_status call_status, gn_call_info *call_info, struct gn_statemachine *state, void *callback_data);
+	gn_error (*on_sms)(gn_sms *message, struct gn_statemachine *state, void *callback_data);
+
+	/* callback local data */
+	void *cb_callback_data;	/* to be passed as callback_data to on_cell_broadcast */
+	void *call_callback_data;	/* to be passed as callback_data to call_notification */
+	void *sms_callback_data;	/* to be passed as callback_data to on_sms */
+} nk6510_driver_instance;
+
 #endif  /* _gnokii_phones_nk6510_h */
