@@ -1,6 +1,6 @@
 /*
 
-  $Id: xgnokii.c,v 1.95 2007/11/21 12:41:27 dforsi Exp $
+  $Id: xgnokii.c,v 1.97 2008/01/11 19:34:57 pkot Exp $
   
   X G N O K I I
 
@@ -22,7 +22,7 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999      Pavel Jank ml., Hugh Blemings
+  Copyright (C) 1999      Pavel Janik ml., Hugh Blemings
   Copyright (C) 1999-2005 Jan Derfinak
   Copyright (C) 2001-2004 Pawel Kot
   Copyright (C) 2001      Chris Kemp, Marian Jancar
@@ -139,7 +139,7 @@ char folders[GN_SMS_FOLDER_MAX_NUMBER][GN_SMS_MESSAGE_MAX_NUMBER];
 gint foldercount = 0, lastfoldercount = 0;
 
 /* Local variables */
-static char *DefaultXGnokiiDir = XGNOKIIDIR "/share/xgnokii";
+static char *DefaultXGnokiiDir = PREFIX "/share/xgnokii";
 static bool SMSSettingsInitialized = FALSE;
 static bool CallersGroupsInitialized = FALSE;
 static gint hiddenCallDialog;
@@ -2359,7 +2359,7 @@ static void ReadConfig(void)
 	if (!xgnokiiConfig.bindir)
 		xgnokiiConfig.bindir = gn_lib_cfg_get("gnokiid", "bindir");
 	if (!xgnokiiConfig.bindir)
-		xgnokiiConfig.bindir = g_strdup("/usr/local/sbin");
+		xgnokiiConfig.bindir = g_strdup(SBINDIR);
 	if (gn_lib_cfg_get("xgnokii", "allow_breakage"))
 		xgnokiiConfig.allowBreakage = atoi(gn_lib_cfg_get("xgnokii", "allow_breakage"));
 	else
