@@ -1,6 +1,6 @@
 /*
 
-  $Id: xgnokii_lowlevel.c,v 1.102 2007/11/19 17:26:18 dforsi Exp $
+  $Id: xgnokii_lowlevel.c,v 1.104 2008/01/04 18:59:40 pkot Exp $
   
   X G N O K I I
 
@@ -22,7 +22,7 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999      Pavel Jank ml., Hugh Blemings
+  Copyright (C) 1999      Pavel Janik ml., Hugh Blemings
   Copyright (C) 1999-2005 Jan Derfinak
   Copyright (C) 2002-2004 Pawel Kot, BORBELY Zoltan
   Copyright (C) 2002      Markus Plail, Manfred Jonsson, Pavel Machek
@@ -30,6 +30,8 @@
 
 */
 
+#include "config.h"
+
 #ifndef WIN32
 #  include <unistd.h>
 #endif
@@ -625,6 +627,9 @@ static gint A_GetCalendarNoteAll(gpointer data)
 
 	gn_data_clear(&gdat);
 
+	memset(&entry, 0, sizeof(entry));
+	memset(&list, 0, sizeof(list));
+
 	pthread_mutex_lock(&calendarMutex);
 	while (1) {
 		entry.location = i;
